from schematics.models import Model
from schematics.types import StringType
from os import path

CONFIGFILE_PATH = path.expanduser(path.join("~", ".jissue"))


class Configuration(Model):
    fqdn = StringType(required=True)
    username = StringType(required=True)
    password = StringType(required=True)

    @classmethod
    def from_file(cls, filepath=None):
        from json import load
        filepath = filepath or CONFIGFILE_PATH
        with open(filepath) as fd:
            return cls(**load(fd))

    def save(self, filepath=None):
        from json import dump
        filepath = filepath or CONFIGFILE_PATH
        with open(filepath, 'w') as fd:
            dump(self.to_python(), fd, indent=4)

    def to_json(self, indent=False):
        from json import dumps
        return dumps(self.to_python(), indent=indent)



from infi.pyutils.lazy import cached_function
from jira.exceptions import JIRAError
from bunch import Bunch
CURRENT_USER = "currentUser()"
ASSIGNED_ISSUES = "{}assignee = {} AND resolution = unresolved ORDER BY priority DESC, created ASC"


@cached_function
def get_jira():
    from .config import Configuration
    from jira.client import JIRA
    config = Configuration.from_file()
    options = dict(server="http://{0}".format(config.fqdn))
    basic_auth = (config.username, config.password)
    return JIRA(options, basic_auth)


@cached_function
def get_issues__assigned_to_user(user, project=None):
    return get_jira().search_issues(ASSIGNED_ISSUES.format("project={} AND ".format(project) if project else '', user))


@cached_function
def get_issues__assigned_to_me(project=None):
    return get_issues__assigned_to_user(CURRENT_USER, project)


def add_labels_to_issue(key, labels):
    issue = get_issue(key)
    labels = set.union(set([unicode(label) for label in labels]), set(issue.fields().labels))
    issue.update(labels=list(labels))


def assign_issue(key, assignee):
    get_jira().assign_issue(key, assignee)


def from_jira_formatted_datetime(formatted_string):
    # http://stackoverflow.com/questions/127803/how-to-parse-iso-formatted-date-in-python
    import re
    import datetime
    return datetime.datetime(*map(int, re.split('[^\d]', formatted_string)[:-1]))


def from_jira_formatted_date(formatted_string):
    return from_jira_formatted_datetime(formatted_string+'T00:00:00.000+0000')


def matches(str_a, str_b):
    return str_a is not None and str_b is not None and str_a.lower() == str_b.lower()


def transition_issue(key, transition_string, fields):
    jira = get_jira()
    issue = jira.issue(key)
    [transition] = [item['id'] for item in jira.transitions(issue) if matches(item['name'], transition_string)]
    jira.transition_issue(issue=issue.key, transitionId=transition, fields=fields)


def resolve_issue(key, resolution_string, fix_versions_strings):
    jira = get_jira()
    issue = jira.issue(key)
    [resolution] = [item.id for item in jira.resolutions() if matches(item.name, resolution_string)]
    project_versions = jira.project_versions(issue.fields().project)
    fix_versions = [dict(id=item.id) for item in project_versions if item.name in fix_versions_strings]
    fields = dict(resolution=dict(id=resolution), fixVersions=fix_versions)
    transition_issue(key, "Resolve Issue", fields)


def start_progress(key):
    transition_issue(key, "Start Progress", dict())


def stop_progress(key):
    transition_issue(key, "Stop Progress", dict())


def reopen(key):
    transition_issue(key, "Reopen Issue", dict())


@cached_function
def get_project(key):
    return get_jira().project(key.upper())


@cached_function
def get_next_release_name_for_issue(key):
    jira = get_jira()
    issue = jira.issue(key)
    project = issue.fields().project.key
    return get_next_release_name_in_project(project)


@cached_function
def get_next_release_name_in_project(key):
    project = get_project(key)
    return sorted([version for version in project.versions if not version.released],
                   key=lambda version: from_jira_formatted_date(getattr(version, "releaseDate", '2121-12-12')))[0].name


def create_issue(project_key, issue_type_name, component_name, fix_version_name, details):
    jira = get_jira()
    project = jira.project(project_key)
    [issue_type] = [issue_type for issue_type in project.issueTypes
                                    if matches(issue_type.name, issue_type_name)]
    components = [component for component in project.components
                                if matches(component.name, component_name)]
    versions = [version for version in project.versions
                            if matches(version.name, fix_version_name)]
    summary = details.split("\n", 1)[0]
    description = details.split("\n", 1)[1:]
    fields = dict(project=dict(id=str(project.id)),
                                issuetype=dict(id=str(issue_type.id)),
                                components=[dict(id=str(component.id)) for component in components],
                                fixVersions=[dict(id=str(version.id)) for version in versions],
                                summary=summary, description=description[0] if description else '')
    issue = jira.create_issue(fields=fields)
    return issue


def create_link(link_type_name, from_key, to_key):
    jira = get_jira()
    [link_type] = [link_type for link_type in jira.issue_link_types()
                                 if matches(link_type.name, link_type_name)]

    kwargs = dict(type=link_type.name, inwardIssue=from_key, outwardIssue=to_key)
    jira.create_issue_link(**kwargs)


def search_issues(query):
    jira = get_jira()
    return jira.search_issues(query, maxResults=100)


def comment_on_issue(key, message):
    get_jira().add_comment(issue=key, body=message)


@cached_function
def get_issue(key):
    return get_jira().issue(key.upper())


@cached_function
def get_query_by_filter(name):
    """:returns: the jql of the filter"""
    for _filter in get_jira().favourite_filters():
        if _filter.name == name:
            return _filter.jql
    raise JIRAError(404, "no such filter")


issue_mappings = Bunch(Rank=lambda issue: int(issue.fields().customfield_10700),
                       Type=lambda issue: issue.fields().issuetype.name,
                       Key=lambda issue: issue.key,
                       Summary=lambda issue: issue.fields().summary,
                       Description=lambda issue: issue.fields().description,
                       Priority=lambda issue: issue.fields().priority.name,
                       Project=lambda issue: issue.fields().project.name,
                       Status=lambda issue: issue.fields().status.name,
                       Resolution=lambda issue: (issue.fields().resolution or Bunch(name="Unresolved")).name,
                       Created=lambda issue: from_jira_formatted_datetime(issue.fields().created),
                       Updated=lambda issue: from_jira_formatted_datetime(issue.fields().updated),
                       Assignee=lambda issue: issue.fields().assignee.displayName,
                       Reporter=lambda issue: issue.fields().reporter.displayName,
                       Labels=lambda issue: issue.fields().labels,
                       Comments=lambda issue: issue.fields().comment.comments,
                       AffectsVersions=lambda issue: [item.name for item in issue.fields().versions],
                       FixVersions=lambda issue: [item.name for item in issue.fields().fixVersions],
                       Components=lambda issue: [item.name for item in issue.fields().components],
                       IssueLinks=lambda issue: issue.fields().issuelinks,
                       SubTasks=lambda issue: issue.fields().subtasks,
                       )



"""jissue
infinidat jira issue command-line tool

Usage:
    jissue list {project} [--sort-by=<column-name>] [--reverse] [--assignee=<assignee>]
    jissue search [--sort-by=<column-name>] [--reverse] (--filter=<filter> | <query>)
    jissue start {issue}
    jissue stop {issue}
    jissue show {issue}
    jissue reopen {issue}
    jissue create <issue-type> <details> {project} [--component=<component>] [--fix-version=<version>] [--short]
    jissue comment <message> {issue}
    jissue commit [<message>] {issue} [--file=<file>...]
    jissue resolve {issue} [--resolve-as=<resolution>] [--fix-version=<version>]
    jissue link <link-type> <target-issue> {issue}
    jissue label {issue} --label=<label>...
    jissue assign {issue} (--assignee=<assignee> | --automatic | --to-no-one | --to-me)
    jissue inventory {project}
    jissue filters
    jissue config show
    jissue config set <fqdn> <username> <password>

Options:
    <project>                    project key {project_default}
    <issue>                      issue key {issue_default}
    <details>                    multiline-string, first line is summary, other is description
    <link-type>                  link type string [default: Duplicate]
    <target-issue>               target issue
    <issue-type>                 issue type string
    --component=<component>      component name {component_default}
    --fix-version=<version>      version string {version_default}
    --file=<file>...             files/directories to commit
    --resolve-as=<resolution>    resolution string [default: Fixed]
    --sort-by=<column-name>      column to sort by [default: Rank]
    --assignee=<assignee>        jira user name
    --filter=<filter>            name of a favorite filter
    --short                      print just the issue key, useful for scripting
    --help                       show this screen
"""


def _get_arguments(argv, environ):
    from .__version__ import __version__
    from docopt import docopt
    from bunch import Bunch
    project_default = "[default: {}]".format(environ["JISSUE_PROJECT"]) if "JISSUE_PROJECT" in environ else ""
    version_default = "[default: {}]".format(environ["JISSUE_VERSION"]) if "JISSUE_VERSION" in environ else ""
    component_default = "[default: {}]".format(environ["JISSUE_COMPONENT"]) if "JISSUE_COMPONENT" in environ else ""
    issue_default = "[default: {}]".format(environ["JISSUE_ISSUE"]) if "JISSUE_ISSUE" in environ else ""
    doc_with_defaults = __doc__.format(project_default=project_default, version_default=version_default,
                                       component_default=component_default, issue_default=issue_default,
                                       issue="[<issue>]" if issue_default else "<issue>",
                                       project="[<project>]" if project_default else "<project>")
    arguments = Bunch(docopt(doc_with_defaults, argv=argv, help=True, version=__version__))
    if environ.get("JISSUE_PROJECT") and not arguments.get("<project>"):
        arguments["<project>"] = environ["JISSUE_PROJECT"]
    if environ.get("JISSUE_VERSION") and not arguments.get("--fix-version"):
        arguments["--fix-version"] = environ["JISSUE_VERSION"]
    if environ.get("JISSUE_COMPONENT") and not arguments.get("<component>"):
        arguments["<component>"] = environ["JISSUE_COMPONENT"]
    if environ.get("JISSUE_ISSUE") and not arguments.get("<issue>"):
        arguments["<issue>"] = environ["JISSUE_ISSUE"]
    return arguments


def _jissue(argv, environ=dict()):
    from sys import stderr
    from copy import deepcopy
    from jira.exceptions import JIRAError
    from infi.execute import ExecutionError
    from .actions import choose_action
    from docopt import DocoptExit
    try:
        arguments = _get_arguments(argv, dict(deepcopy(environ)))
        action = choose_action(argv)
        return action(arguments) or 0
    except DocoptExit, e:
        print >> stderr, e
        return 1
    except SystemExit, e:
        print >> stderr, e
        return 0
    except JIRAError, e:
        print >> stderr, e
    except ExecutionError, e:
        print >> stderr, e.result.get_stderr()
    return 1


def main():
    from os import environ
    from sys import argv
    return _jissue(argv[1:], environ)



from __future__ import print_function


def format(value, slice=None):
    from datetime import datetime
    from jira.resources import Comment, Issue, IssueLink
    if isinstance(value, datetime):
        return value.strftime("%Y-%m-%d %H:%M")
    if isinstance(value, (list, tuple)):
        if len(value) and isinstance(value[0], (Comment, )):
            from .jira_adapter import from_jira_formatted_datetime
            return "\n\n".join([u"{0} added a comment - {1}\n{2}".format(item.author.displayName,
                                                                         format(from_jira_formatted_datetime(item.created)),
                                                                         item.body)
                                for item in value])
        if len(value) and isinstance(value[0], (IssueLink, )):
            from .jira_adapter import issue_mappings
            get_linked_issue = lambda item: getattr(item, "inwardIssue", getattr(item, "outwardIssue", None))
            get_link_text = lambda item: item.type.inward if hasattr(item, "inwardIssue") else item.type.outward
            return "\n\n".join(["u{0:<20} {1:<15} {2:<15} {3}".format(get_link_text(item),
                                                                      issue_mappings['Key'](get_linked_issue(item)),
                                                                      issue_mappings['Status'](get_linked_issue(item)),
                                                                      issue_mappings['Summary'](get_linked_issue(item)))
                                for item in value])
        if len(value) and isinstance(value[0], (Issue, )):
            from .jira_adapter import issue_mappings
            return "\n".join(["u{0:<20} {1:<15} {2:<15} {3}".format('',
                                                                    issue_mappings['Key'](item),
                                                                    issue_mappings['Status'](item),
                                                                    issue_mappings['Summary'](item))
                              for item in value])
        return ', '.join(value)
    return unicode(value)[:slice]


def _list_issues(arguments, issues):
    from prettytable import PrettyTable
    from .jira_adapter import from_jira_formatted_datetime, issue_mappings
    table = PrettyTable(["Rank", "Type", "Key", "Summary", "Status", "Created", "Updated"])
    table.align = 'l'
    sortby_column = arguments.get("--sort-by").capitalize()
    reverse = arguments.get("--reverse")
    for issue in issues:
        table.add_row([issue_mappings[column](issue) for column in table.field_names])
    print(table.get_string(reversesort=reverse, sortby=sortby_column, align='l'))


def list_issues(arguments):
    from .jira_adapter import get_issues__assigned_to_me, get_issues__assigned_to_user
    user = arguments.get("--assignee")
    project = arguments.get("<project>")
    issues = get_issues__assigned_to_user(user, project) if user else get_issues__assigned_to_me(project)
    _list_issues(arguments, issues)


def search(arguments):
    from .jira_adapter import search_issues, get_query_by_filter
    query = arguments.get("<query>")
    _filter = arguments.get("--filter")
    if _filter:
        query = get_query_by_filter(_filter)
    return _list_issues(arguments, search_issues(query))


def start(arguments):
    from .jira_adapter import start_progress
    start_progress(arguments.get("<issue>"))


def stop(arguments):
    from .jira_adapter import stop_progress
    stop_progress(arguments.get("<issue>"))


def get_issue_pretty(key):
    from textwrap import dedent
    from string import printable
    template = u"""
    {Project} / {Key}
    {Summary}

    Type:              {Type:<11}                       Status:        {Status:<15}            Assignee: {Assignee:<15}
    Priority:          {Priority:<15}                   Resolution:    {Resolution:<19}        Reporter: {Reporter}
    Affects Version/s: {AffectsVersions:<22}            Fix Version/s: {FixVersions:<26}
    Components: {Components:<59}                        Created: {Created}
    Labels: {Labels:<55}                                Updated: {Updated}

    Issue Links:
    {IssueLinks}

    Sub-Tasks:
    {SubTasks}

    Description:
    {Description}

    Comments:
    {Comments}
    """
    keywords = ["Project", "Key", "Summary", "Type", "Status",
                "Priority", "Resolution", "Assignee", "Reporter",
                "AffectsVersions", "FixVersions", "Components",
                "Created", "Updated", "Labels",
                "Description", "Comments", "IssueLinks", "SubTasks"]
    from .jira_adapter import get_issue, issue_mappings
    issue = get_issue(key)
    kwargs = {item: format(issue_mappings[item](issue)) for item in keywords}
    data = dedent(template).format(**kwargs)
    data = ''.join([item for item in data if item in printable])
    return data


def show(arguments):
    print(get_issue_pretty(arguments.get("<issue>")))


def comment(arguments):
    from .jira_adapter import comment_on_issue
    comment_on_issue(arguments.get("<issue>"), arguments.get("<message>"))


def resolve(arguments):
    from .jira_adapter import resolve_issue, get_next_release_name_for_issue
    from string import capwords
    key = arguments.get("<issue>")
    fix_version = arguments.get("--fix-version") or get_next_release_name_for_issue(key)
    resolution = capwords(arguments.get("--resolve-as"))
    resolve_issue(key, resolution, [fix_version])
    print("{0} resolved in version {1}".format(key, fix_version))


def link(arguments):
    from .jira_adapter import create_link
    from string import capwords
    create_link(capwords(arguments.get("<link-type>")), arguments.get("<issue>"), arguments.get("<target-issue>"))


def create(arguments):
    from .jira_adapter import create_issue, get_next_release_name_in_project
    from string import capwords
    project_key = arguments.get("<project>").upper()
    details = arguments.get("<details>")
    description = arguments.get("<description>")
    component_name = arguments.get("--component") or None
    issue_type_name = capwords(arguments.get("<issue-type>"))
    fix_version_name = arguments.get("--fix-version") or get_next_release_name_in_project(project_key)
    component_name = arguments.get("--component")
    issue = create_issue(project_key, issue_type_name, component_name, fix_version_name, details)
    print(issue.key) if arguments.get("--short") else show({"<issue>": issue.key})
    return issue.key


def assign(arguments):
    from .jira_adapter import assign_issue
    from .config import Configuration
    key = arguments.get("<issue>")
    assignee = arguments.get("--assignee") if arguments.get("--assignee") else \
        "-1" if arguments.get("--automatic") else \
        Configuration.from_file().username if arguments.get("--to-me") else None  # --to-no-one
    assign_issue(key, assignee)


def config_show(arguments):
    from .config import Configuration
    try:
        print(Configuration.from_file().to_json(indent=True))
    except IOError:
        print("Configuration file does not exist")


def config_set(arguments):
    from .config import Configuration

    values = {item: getattr(arguments, "<{0}>".format(item))
              for item in ['fqdn', 'username', 'password']}
    config = Configuration(**values)
    config.save()


def inventory(arguments):
    from .jira_adapter import get_jira
    from string import capwords
    from pprint import pprint
    project_key = arguments.get("<project>")
    jira = get_jira()
    component_names = [item.name for item in jira.project_components(project_key)]
    unreleased_versions = [item.name for item in jira.project_versions(project_key) if not item.released]
    resolution_names = [item.name for item in jira.resolutions()]
    pprint({"Components": component_names, "Versions (unreleased)": unreleased_versions,
            "Resolve types": resolution_names})


def label(arguments):
    from .jira_adapter import add_labels_to_issue
    add_labels_to_issue(arguments.get("<issue>"), arguments.get("--label"))


def reopen(arguments):
    from .jira_adapter import reopen
    reopen(arguments.get("<issue>"))


def commit(arguments):
    from .jira_adapter import get_issue
    from .config import Configuration
    from infi.execute import execute_assert_success
    from sys import stdin, stdout, stderr
    from subprocess import Popen

    args = ["git", "commit"] + arguments.get("--file")
    message = arguments.get("<message>") or ''
    key = arguments.get("<issue>")
    data = get_issue_pretty(key)
    username = Configuration.from_file().username
    shame = '@{} why you no put commit message'.format(username)
    args += ["--message", "{} {}".format(key, message if message else shame),
             "--message", '='*80 + '\n' + data]
    if message:
        execute_assert_success(args)
    else:
        args += ["--edit"]
        Popen(args, stdout=stdout, stderr=stderr, stdin=stdin).wait()


def filters(arguments):
    from .jira_adapter import get_jira
    from prettytable import PrettyTable
    table = PrettyTable(["name", "query"])
    table.align = 'l'
    for _filter in get_jira().favourite_filters():
        table.add_row([_filter.name, _filter.jql])
    print(table)


def get_mappings():
    return dict(
        list=list_issues,
        start=start,
        stop=stop,
        show=show,
        create=create,
        comment=comment,
        resolve=resolve,
        link=link,
        inventory=inventory,
        assign=assign,
        search=search,
        label=label,
        commit=commit,
        reopen=reopen,
        filters=filters,
        config=dict(show=config_show, set=config_set),
    )


def choose_action(argv):
    argv = argv[::-1]
    mappings = get_mappings()
    while True:
        keyword = argv.pop()
        mappings = mappings[keyword]
        if hasattr(mappings, "__call__"):
            return mappings



"""jish
Usage:
    jish project <project> [<component> [<version>]]
    jish component <component>
    jish version <version>
    jish workon <issue>
    jish create <issue-type> <details>
    jish deactivate

 Options:
    --help   this screen
"""

from sys import stderr, stdout
from . import jira_adapter
from os import environ


def _get_arguments(argv):
    from .__version__ import __version__
    from docopt import docopt
    from bunch import Bunch
    arguments = Bunch(docopt(__doc__, argv=argv, help=True, version=__version__))
    return arguments


def clear_environment_variables(environment_variables):
    issue_active = bool(environ.get("JISSUE_ISSUE"))
    if not issue_active:
        environment_variables.update(JISSUE_PROJECT="", JISSUE_VERSION="", JISSUE_COMPONENT="")
    environment_variables.update(JISSUE_ISSUE="")


def set_environment_variables_for_issue(arguments, environment_variables):
    issue_key = arguments.get("<issue>") or environ.get("JISSUE_ISSUE")
    try:
        jira_adapter.get_issue(issue_key or "_")
    except jira_adapter.JIRAError:
        print >> stderr, "no such issue", issue_key
        raise SystemExit(1)
    environment_variables.update(JISSUE_ISSUE=issue_key.upper())


def set_environment_variables_for_project(arguments, environment_variables):
    project_key = arguments.get("<project>") or environ.get("JISSUE_PROJECT")
    try:
        project = jira_adapter.get_project(project_key or "_")
    except jira_adapter.JIRAError:
        print >> stderr, "no such project", project_key
        raise SystemExit(1)
    next_version = jira_adapter.get_next_release_name_in_project(project_key)
    project_version = arguments.get("<version>") if not arguments.project else next_version \
                      or (environ.get("JISSUE_VERSION") if not arguments.project else '') \
                      or next_version
    if project_version not in [item.name for item in project.versions]:
        print >> stderr, "no such version", project_version
        raise SystemExit(1)
    project_component = arguments.get("<component>") or environ.get("JISSUE_COMPONENT")
    if project_component not in [item.name for item in project.components] + [None, '']:
        print >> stderr, "no such component", project_component
        raise SystemExit(1)
    environment_variables.update(JISSUE_PROJECT=project_key.upper(), JISSUE_VERSION=project_version)
    if project_component:
        environment_variables.update(JISSUE_COMPONENT=project_component)


def set_environment_variables(arguments, environment_variables):
    if arguments.workon:
        return set_environment_variables_for_issue(arguments, environment_variables)
    return set_environment_variables_for_project(arguments, environment_variables)


def _jish(argv):
    try:
        arguments = _get_arguments(argv)
    except SystemError, e:
        print >> stderr, e
        return 0
    environment_variables = dict()
    if arguments.deactivate:
        clear_environment_variables(environment_variables)
    elif arguments.project or arguments.component or arguments.version or arguments.workon:
        set_environment_variables(arguments, environment_variables)
    elif arguments.create:
        args = (environ['JISSUE_PROJECT'],
                arguments.get("<issue-type>"),
                environ.get('JISSUE_COMPONENT') or None,
                environ.get('JISSUE_VERSION'),
                arguments.get("<details>"))
        arguments.workon = True
        arguments['<issue>'] = jira_adapter.create_issue(*args).key
        set_environment_variables(arguments, environment_variables)
    for key, value in environment_variables.iteritems():
        print >> stdout, "export {}={}\n".format(key, value)


def main():
    from sys import argv
    return _jish(argv[1:])



__version__ = "0.1.3"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(detached from v0.1.3)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '12766fca54d65676f8a5d9a801d51cfd45e9062e'
__git_head_subject__ = 'HOSTDEV-944 fixing version resolution when switching between projects'
__git_head_message__ = '================================================================================\n\nHost Infrastructure / HOSTDEV-944\njish project <p> <c> fails with no such version if swtiched from other project\n\nType:              Bug                               Status:        Open                       Assignee: Guy Rozendorn\nPriority:          Minor                             Resolution:    Unresolved                 Reporter: Guy Rozendorn\nAffects Version/s:                                   Fix Version/s: 0.30\nComponents: jissue                                                                             Created: 2013-06-09 12:51\nLabels:                                                                                        Updated: 2013-06-27 21:02\n\nIssue Links:\n\nSub-Tasks:\n\nDescription:\nNone\n\nComments:'
__git_dirty_diff__ = ''



SETUP_INFO = dict(
    name = 'infi.jira_cli',
    version = '0.1.3',
    author = 'Guy Rozendorn',
    author_email = 'guy@rzn.co.il',

    url = 'https://infinigit.infinidat.com/host/infi-jira-cli',
    license = 'PSF',
    description = """JIRA command-line tools""",
    long_description = """JIRA command-line tools""",

    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Python Software Foundation License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

    install_requires = ['bunch==1.0.0', 'git-py>=1.0.0', 'PrettyTable', 'infi.execute>=0.0.12', 'docopt>=0.6.1', 'infi.pyutils>=0.0.30', 'infi.docopt-completion>=0.1.4', 'jira-python>=0.13', 'infi.recipe.console-scripts>=0.3.4', 'distribute>=0.6.45', 'schematics>=0.5'],
    namespace_packages = ['infi'],

    package_dir = {'': 'src'},
    package_data = {'': ['jish.zsh']},
    include_package_data = True,
    zip_safe = False,

    entry_points = dict(
        console_scripts = ['setup-jish = infi.jira_cli.setup:main', 'jissue = infi.jira_cli:main'],
        gui_scripts = [],
        ),
)

if SETUP_INFO['url'] is None:
    _ = SETUP_INFO.pop('url')

def setup():
    from setuptools import setup as _setup
    from setuptools import find_packages
    SETUP_INFO['packages'] = find_packages('src')
    _setup(**SETUP_INFO)

if __name__ == '__main__':
    setup()




__import__("pkg_resources").declare_namespace(__name__)

from os import environ
from . import jissue, jish


def main():
    call_jish = int(environ.get("jish", "0"))
    return jish.main() if call_jish else jissue.main()
